#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""Tests for `cypyler` package."""

import pytest

from cypyler import TMPCypyler


def test_cypyler():
    code = """def add(x, y): return x + y"""

    cp = TMPCypyler()
    built_module = cp.build(code)

    assert built_module.add(2, 3) == 5



import importlib

from distutils.core import Distribution, Extension
from distutils.command.build_ext import build_ext

from Cython.Build import cythonize


def _get_build_extension(extension, lib_dir):
    dist = Distribution()
    config_files = dist.find_config_files()

    # TODO: Is this check really useful?
    try:
        config_files.remove('setup.cfg')
    except ValueError:
        pass

    dist.parse_config_files(config_files)

    build_extension_ = build_ext(dist)
    build_extension_.finalize_options()

    build_extension_.build_temp = lib_dir
    build_extension_.build_lib = lib_dir
    build_extension_.extensions = [extension]

    return build_extension_


def build_extension(extension, lib_dir):
    build_extension_ = _get_build_extension(
        extension, lib_dir=lib_dir)
    build_extension_.run()

    return build_extension_


def cypyle(modname, pyxfilename, libraries=None, include_dirs=None):
    if not libraries:
        libraries = []
    if not include_dirs:
        include_dirs = []

    import numpy as np
    ext = Extension(name=modname,
                    sources=[pyxfilename],
                    include_dirs=[np.get_include()]+include_dirs,
                    language="c++",
                    libraries=libraries)

    opts = dict(
        force=True
    )

    return cythonize([ext], **opts)


def import_extension(module_name, ext_path):
    spec = importlib.util.spec_from_file_location(module_name, ext_path)
    mod = importlib.util.module_from_spec(spec)

    return mod



import os
import tempfile

from .utils import cypyle, build_extension, import_extension


class Cypyler:

    def __init__(self, build_dir, libraries=None, include_dirs=None):
        """ Instatiates a cypyler.
        :param build_dir: Path to the directory where the build will occur.
        :param libraries: List of libraries to pass to Cython.
        :param include_dirs: List of include directories to pass to Cython.
        :type build_dir: str
        :type libraries: list[str]
        :type include_dirs: list[str]
        """
        self.build_dir = build_dir
        self.libraries = libraries
        self.include_dirs = include_dirs

    def build(self, code, suffix='.pyx'):
        # A python module must begin with a underscore or letter, hence the prefix
        module_fd, module_path = tempfile.mkstemp(
            prefix='_', suffix=suffix, dir=self.build_dir)

        module_filename = os.path.basename(module_path)
        module_name = os.path.splitext(module_filename)[0]

        with os.fdopen(module_fd, 'w') as f:
            f.write(code)

        extensions = cypyle(module_name, module_path,
                            include_dirs=self.include_dirs,
                            libraries=self.libraries)

        built_ext = build_extension(extensions[0], self.build_dir)
        ext_path = built_ext.get_ext_fullpath(module_name)

        built_module = import_extension(module_name, ext_path)

        return built_module


class TMPCypyler(Cypyler):

    def __init__(self, build_dir_prefix='', libraries=None, include_dirs=None):
        """ Instatiates a cypyler that uses a temporary directory as build dir.
        :param build_dir_prefix: Prefix to the build directory.
        :param libraries: List of libraries to pass to Cython.
        :param include_dirs: List of include directories to pass to Cython.
        :type build_dir_prefix: str
        :type libraries: list[str]
        :type include_dirs: list[str]
        """
        build_dir = tempfile.mkdtemp(prefix=build_dir_prefix)

        super().__init__(build_dir, libraries, include_dirs)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""The setup script."""

from setuptools import setup, find_packages

with open('README.rst') as readme_file:
    readme = readme_file.read()

setup_requirements = ['pytest-runner', ]

test_requirements = ['pytest', ]

setup(
    author="Guilherme Caminha",
    author_email='gpkc@cin.ufpe.br',
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Natural Language :: English',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.6',
    ],
    description="A cython compiler for compiling source code in string form.",
    install_requires=['cython', 'numpy'],
    license="MIT license",
    long_description=readme,
    include_package_data=True,
    keywords='cypyler',
    name='cypyler',
    packages=find_packages(include=['cypyler']),
    setup_requires=setup_requirements,
    test_suite='tests',
    tests_require=test_requirements,
    url='https://github.com/gpkc/cypyler',
    version='0.2.0',
    zip_safe=False,
)
